/*
 *
 * Copyright (c) 2000 Scott Oaks and Henry Wong. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.rmi.*;
import net.jini.discovery.*;
import net.jini.core.lease.*;
import net.jini.core.event.*;
import com.sun.jini.lease.*;

public class ConvertClient {

    public static void main(String[] args) throws Exception {
        System.setSecurityManager(new RMISecurityManager());

        // Find the service by its interface type
        ServiceFinder sf = new ServiceFinder(ConvertService.class);
        ConvertService cs = (ConvertService) sf.getObject();

        LeaseRenewalManager lrm = new LeaseRenewalManager();

        // Sign up to be notified of all conversions
        RemoteEventListener rel = new ConvertEventHandler();
        EventRegistration er = cs.trackConversions(Lease.FOREVER, rel, null);
        lrm.renewUntil(er.getLease(), Lease.FOREVER, null);

        // Now do a conversion for ourself
        ConvertServiceRegistration csr = cs.getInstance(Lease.FOREVER);
        lrm.renewUntil(csr.getLease(), Lease.FOREVER, null);

        // Now invoke methods on the service object
        System.out.println(csr.convert(5));

        // We'll sit here, waiting forever for other event notifications
        // to happen.
        //try {
         //   Thread.sleep(Integer.MAX_VALUE);
        //} catch (InterruptedException ie) {}

        // If you want to exit, you should clean up the leases first, e.g., 
        lrm.cancel(er.getLease());
        lrm.cancel(csr.getLease());
        System.exit(0);
    }
}
