/*
 *
 * Copyright (c) 2000 Scott Oaks and Henry Wong. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.rmi.*;
import java.rmi.server.*;
import java.io.*;

// This package class is the class definition of a complex number
// Instances of this class (as well as the class itself) will
// be sent to the client when the client calls the getInstance()
// method
class ComplexNumberImpl implements ComplexNumber, Serializable {
    private double real, imaginary;

    public double getReal() {
        return real;
    }

    public double getImaginary() {
        return imaginary;
    }

    public void setReal(double r) {
        real = r;
    }

    public void setImaginary(double i) {
        imaginary = i;
    }

    public String toString() {
        return real + " + " + imaginary + "i";
    }
}

public class ComplexNumberFactoryImpl extends UnicastRemoteObject implements ComplexNumberFactory {

    public ComplexNumberFactoryImpl() throws RemoteException {
    }

    public ComplexNumber getInstance(double r, double i) {
        ComplexNumberImpl cni = new ComplexNumberImpl();
        cni.setReal(r);
        cni.setImaginary(i);
        return cni;
    }

    public static void main(String[] args) throws Exception {
        ComplexNumberFactoryImpl cnfi = new ComplexNumberFactoryImpl();
        Naming.rebind("ComplexNumberFactory", (Remote) cnfi);
    }
}
