/*
 *
 * Copyright (c) 2000 Scott Oaks and Henry Wong. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import net.jini.core.discovery.*;
import net.jini.core.entry.*;
import com.sun.jini.lookup.*;

public class ConvertServiceAdminImpl extends UnicastRemoteObject implements ConvertServiceAdminProxy {
    private JoinManager manager;
    private boolean allowAccess = true;

    public ConvertServiceAdminImpl(JoinManager manager) throws RemoteException {
        this.manager = manager;
    }

    private void checkAccess() {
        if (allowAccess == false)
            throw new SecurityException("Admin access not allowed");
    }

    public Entry[] getLookupAttributes() throws RemoteException {
        checkAccess();
        return manager.getAttributes();
    }

    public void addLookupAttributes(Entry[] attrSets) throws RemoteException {
        checkAccess();
        manager.addAttributes(attrSets, true);
    }

    public void modifyLookupAttributes(Entry[] attrSetTemplates, Entry[] attrSets) throws RemoteException {
        checkAccess();
        manager.modifyAttributes(attrSetTemplates, attrSets, true);
    }

    public String[] getLookupGroups() throws RemoteException {
        checkAccess();
        return manager.getGroups();
    }

    public void addLookupGroups(String[] groups) throws RemoteException {
        checkAccess();
        try {
            manager.addGroups(groups);
        } catch (IOException ex) {
            throw new RemoteException("Error in AddGroups", ex);
        }
    }

    public void removeLookupGroups(String[] groups) throws RemoteException {
        checkAccess();
        try {
            manager.removeGroups(groups);
        } catch (IOException ex) {
            throw new RemoteException("Error in RemoveGroups", ex);
        }
    }

    public void setLookupGroups (String[] groups) throws RemoteException {
        checkAccess();
        try {
            manager.setGroups(groups);
        } catch (IOException ex) {
            throw new RemoteException("Error in SetGroups", ex);
        }
    }

    public LookupLocator[] getLookupLocators() throws RemoteException {
        checkAccess();
        return manager.getLocators();
    }

    public void addLookupLocators(LookupLocator[] locators) throws RemoteException {
        checkAccess();
        manager.addLocators(locators);
    }


    public void removeLookupLocators(LookupLocator[] locators) throws RemoteException {
        checkAccess();
        manager.removeLocators(locators);
    }

    public void setLookupLocators(LookupLocator[] locators) throws RemoteException {
        checkAccess();
        manager.setLocators(locators);
    }

    public void denyAdminAccess() {
        allowAccess = false;
    }

    public void allowAdminAccess() {
        allowAccess = true;
    }
}
